package com.testsound;

/**
 *
 * @author dwazir
 */
public class OrderTwoFilters {

    public static double[] derive(double[] sig) {
        int sz = sig.length;
        double[] ret = new double[sz - 1];
        double prev = sig[0];
        for (int i = 1; i < sz; i++) {
            double x = sig[i];
            ret[i - 1] = prev - x;
        }
        return ret;
    }
    
    public static double[] highPassFilter(double[] array, double cut, double fE) {
        double[] result = null;
        if(array != null) {
            double w = 2.0 * fE;
            cut *= 2.0F * Math.PI;
            double norm = 1.0 / (cut + w);
            double a0 = w * norm;
            double a1 = -a0;
            double b1 = (w - cut) * norm;
            result = new double[array.length];
            for (int i = 1; i < result.length; i++) {
                result[i] = array[i] * a0 + array[i - 1] * a1 + result[i - 1] * b1;
            }
        }
        return result;
    }
    public static double[] highPassFilter(short[] array, double cut, double fE) {
        double[] result = null;
        if(array != null) {
            double w = 2.0 * fE;
            cut *= 2.0F * Math.PI;
            double norm = 1.0 / (cut + w);
            double a0 = w * norm;
            double a1 = -a0;
            double b1 = (w - cut) * norm;
            result = new double[array.length];
            for (int i = 1; i < result.length; i++) {
                result[i] = array[i] * a0 + array[i - 1] * a1 + result[i - 1] * b1;
            }         
        }
        return result;
    }
    public static double[] rejectFilter2(short[] array, double cut, double sampling) {
        return filter_(array, cut, sampling, 1, 0, 1);
    }
    public static double[] rejectFilter2(double[] array, double cut, double sampling) {
        return filter_(array, cut, sampling, 1, 0, 1);
    }
    public static double[] lowPassFilter2(short[] array, double cut, double sampling) {
        return filter_(array, cut, sampling, 1, 0, 0);
    }
    public static double[] lowPassFilter2(double[] array, double cut, double sampling) {
        return filter_(array, cut, sampling, 1, 0, 0);
    }
    public static double[] highPassFilter2(short[] array, double cut, double sampling) {
        return filter_(array, cut, sampling, 0, 0, 1);
    }
    public static double[] bandPassFilter2(short[] array, double cut, double sampling) {
        return filter_(array, cut, sampling, 0, 1, 0);
    }
    public static double[] bandPassFilter2(double[] array, double cut, double sampling) {
        return filter_(array, cut, sampling, 0, 1, 0);
    }
    public static double[] highPassFilter2(double[] array, double cut, double sampling) {
        return filter_(array, cut, sampling, 0, 0, 1);
    }
    private static double[] filter_(double[] array, double cut, double sampling,
            double A0, double A1, double A2) {
        double[] result = null;
        if(array != null) {
            double B0 = 1, B1 = Math.sqrt(2.), B2 = 1;
            double C = 1./Math.atan(Math.PI*cut/sampling);
            double C2 = C*C;
            double A = B0 + B1*C + B2*C2;
            double a0 = (A0 + A1*C + A2*C2)/A;
            double a1 = 2*(A0 - A2*C2)/A;
            double a2 = (A0 - A1*C + A2*C2)/A;
            double b1 = -2*(B0 - B2*C2)/A;
            double b2 = -(B0 - B1*C + B2*C2)/A;
            result = new double[array.length];
            for (int i = 2, n=result.length; i < n; i++) {
                result[i] = array[i] * a0 + 
                            array[i - 1] * a1 +
                            array[i - 2] * a2 +
                            result[i - 1] * b1 +
                            result[i - 2] * b2;
            }         
        }
        return result;
    }
    private static double[] filter2_(double[] array, double cut, double sampling,
            double A0, double A1, double A2) {
        double[] result = null;
        if(array != null) {
            double B0 = 1, B1 = Math.sqrt(2.), B2 = 1;
            double C = 1./Math.atan(Math.PI*cut/sampling);
            double C2 = C*C;
            double A = B0 + (B1 + B2*C)*C;
            double a0 = (A0 + A1*C + A2*C2)/A;
            double a1 = 2*(A0 - A2*C2)/A;
            double a2 = (A0 - A1*C + A2*C2)/A;
            double b1 = -2*(B0 - B2*C2)/A;
            double b2 = -(B0 - B1*C + B2*C2)/A;
            result = new double[array.length];
            for (int i = 2, n=result.length; i < n; i++) {
                result[i] = array[i] * a0 + 
                            array[i - 1] * a1 +
                            array[i - 2] * a2 +
                            result[i - 1] * b1 +
                            result[i - 2] * b2;
            }         
        }
        return result;
    }
    private static double[] filter_(short[] array, double cut, double sampling,
            double A0, double A1, double A2) {
        double[] ret = null;
        if(array != null) {
//            double A0 = 1, A1 = 0, A2 = 0;
//            double A0 = 0, A1 = 0, A2 = 1;
            double B0 = 1, B1 = Math.sqrt(2.), B2 = 1;
            double C = 1./Math.atan(Math.PI*cut/sampling);
            double C2 = C*C;
            double A = B0 + B1*C + B2*C2;
            double a0 = (A0 + A1*C + A2*C2)/A;
            double a1 = (2*A0 - 2*A2*C2)/A;
            double a2 = (A0 - A1*C + A2*C2)/A;
            double b1 = (2*B0 - 2*B2*C2)/A;
            double b2 = (B0 - B1*C + B2*C2)/A;
            double[] result = new double[array.length];
            double xi, xi_1, xi_2;
            double xxi, xxi_1, xxi_2;
            double ri_1, ri_2;
            for (int i = 2, n=result.length; i < n; i++) {
                xi = array[i];
                xi_1 = array[i-1];
                xi_2 = array[i-2];
                ri_1 = result[i - 1];
                ri_2 = result[i - 2];
                /*xxi = xi*xi;
                xxi_1 = xi_1*xi_1;
                xxi_2 = xi_2*xi_2;
                if(xi<0)
                    xxi = -xxi;
                if(xi_1<0)
                    xxi_1 = -xxi_1;
                if(xi_2<0)
                    xxi_2 = -xxi_2;*/
                //ri_1 *= ri_1;
                //ri_2 *= ri_2;
                
                result[i] = /*Math.sqrt*/(a0*xi + a1*xi_1 + a2*xi_2 - b1*ri_1 - b2*ri_2);
            }
            //ret = Arrays.copyOf(result, result.length - 2);
            /*for (int i = 0, n=result.length; i < n; i++) {
                result[i] = Math.sqrt(result[i]);
            }*/
            /*
            double xi, xi_1, xi_2;
            double ri_1, ri_2;
            for (int i = 2, n=result.length; i < n; i++) {
                xi = array[i];
                xi_1 = array[i-1];
                xi_2 = array[i-2];
                ri_1 = result[i - 1];
                ri_2 = result[i - 2];
                result[i] = (
                            a0*xi + a1*xi_1 * a2*xi_2
                            - b1*ri_1 - b2*ri_2);
            }*/        
            /*for (int i = 2; i < result.length; i++) {
                result[i] = array[i] * a0 + 
                            array[i - 1] * a1 +
                            array[i - 2] * a2 -
                            result[i - 1] * b1 -
                            result[i - 2] * b2;
            }*/    
        }
        return ret;
    }
    
}
